﻿
using System;
using System.IO;
using System.Drawing;
using System.Threading;
using System.Drawing.Imaging;
using System.Threading.Tasks;
using System.Collections.Generic;

namespace exc.jdbi.Pixelmanipulation
{
  public partial class PixelManipulation
  {
    public Point[] FindColor(string srcfile, Color col, bool _first = true)
    {
      var fi = new FileInfo(srcfile);
      if (!fi.Exists)
        throw new FileNotFoundException(nameof(srcfile));

      var bm = ToBitmapFile(srcfile);
      var result = new List<Point>();
      unsafe
      {
        var bpp = Image.GetPixelFormatSize(bm.PixelFormat) / 8;
        var rc = new Rectangle(0, 0,
                               bm.Width,
                               bm.Height);

        BitmapData bmdata = bm.LockBits(rc,
                             ImageLockMode.ReadWrite,
                             bm.PixelFormat);

        byte* ptrscan = (byte*)bmdata.Scan0.ToPointer();

        var cts = new CancellationTokenSource();
        var token = cts.Token;
        var options = new ParallelOptions() { CancellationToken = token };

        try
        {
          var redc = col.R;
          var bluec = col.B;
          var greenc = col.G;
          var alphac = col.A;

          Parallel.For(0, bmdata.Height, options,
            y =>
            {
              if (options.CancellationToken.IsCancellationRequested)
                return;

              byte* currentline = ptrscan + (y * bmdata.Stride);
              for (int x = 0; x < (bpp * bmdata.Width); x += bpp)
              {
                //BGR >> Blue(x), Green(x+1), Red(x+2)
                if (currentline[x] == bluec)
                {
                  if (currentline[x + 1] == greenc)
                  {
                    if (currentline[x + 2] == redc)
                    {
                      var bol = false;
                      if (4 == bpp)
                      {
                        if (currentline[x + 3] == alphac)
                          bol = true;
                      }
                      else bol = true;
                      if (bol)
                      {
                        result.Add(new Point(x / bpp, y));
                        if (_first)
                        {
                          cts.Cancel();
                          return;
                        } 
                      }
                    }
                  }
                }
              }
            });
        }
        catch (OperationCanceledException) { }
        catch (Exception) { }

        bm.UnlockBits(bmdata);
        if (!token.IsCancellationRequested)
          cts.Cancel();
        cts.Dispose();
      }
      return result.ToArray();
    }
  }
}