﻿
Option Strict On
Option Explicit On

Imports System.ComponentModel
Imports System.Security.Cryptography
Imports System.Runtime.InteropServices

Namespace exc.jdbi.vb.Randoms
   ''' <summary>
   ''' Simple Int64/Int32 Crypto-Random
   ''' </summary>
   Friend NotInheritable Class SOCR
      ''' <summary>
      ''' Stellt sicher, dass die WarmUp-Phase nur
      ''' einmal beim Starten des Programmes durch-
      ''' laufen wird.
      ''' </summary>
      Private Shared FirstWarmUp As Boolean = True

      ''' <summary>
      ''' RandomNumberGenerator Variable.
      ''' </summary>
      <Browsable(False)>
      <DebuggerBrowsable(DebuggerBrowsableState.Never)>
      <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)>
      Public Shared _Rand As RandomNumberGenerator = RandomNumberGenerator.Create()

      ''' <summary>
      ''' Fill a Array of Byte with Random-Bytes
      ''' without Zero
      ''' </summary>
      ''' <param name="bytes">Byte-Array</param>
      Friend Shared Sub NextBytes(ByVal bytes As Byte())
         If FirstWarmUp Then WarmUp()
         _Rand.GetNonZeroBytes(bytes)
      End Sub

      ''' <summary>
      ''' Next Random Int32 (without Exceptions)
      ''' </summary>
      ''' <returns>Return a Random Int32</returns>
      Friend Shared Function [Next]() As Int32
         ' Crypto-random number of Int32
         If FirstWarmUp Then WarmUp()
         Dim scale As Int32 = -1
         Dim b As Byte() = Nothing
         Dim sz = Marshal.SizeOf(GetType(Int32))
         While scale < 0
            b = New Byte(sz - 1) {}
            _Rand.GetNonZeroBytes(b)
            scale = b(0) Or b(1) << 8I Or b(2) << 16I Or b(3) << 24I
         End While
         Return Math.Abs(scale)
      End Function


      ''' <summary>
      ''' Random Int32 in Range min - max (without Exceptions)
      ''' </summary>
      ''' <param name="_min">Least legal value for the Random number.</param>
      ''' <param name="_max">The greatest legal return value.</param>
      ''' <returns>Return a Int32 [min - max]</returns>
      Friend Shared Function [Next](ByVal _min As Int32, ByVal _max As Int32) As Int32
         ' Crypto-random number of Int32 with min - max
         If _min = _max Then Return _min
         SwapCheck(_min, _max)
         Dim scale As Int32 = [Next]()
         Return Math.Abs(CInt(_min + (_max - _min) * (scale / Int32.MaxValue)))
      End Function


      ''' <summary>
      ''' Next Random Int64 (without Exceptions)
      ''' </summary>
      ''' <returns>Return a Random Int64</returns>
      Friend Shared Function NextI64() As Int64
         ' Crypto-random number of Int64
         If FirstWarmUp Then WarmUp()
         Dim scale As Int64 = -1
         Dim b As Byte() = Nothing
         Dim sz = Marshal.SizeOf(GetType(Int64))
         While scale < 0
            b = New Byte(sz - 1) {}
            _Rand.GetNonZeroBytes(b)
            'scale = Convert.ToInt64(b(0)) Or Convert.ToInt64(b(1)) << 8 Or Convert.ToInt64(b(2)) << 16 Or Convert.ToInt64(b(3)) << 24 Or Convert.ToInt64(b(4)) << 32 Or Convert.ToInt64(b(5)) << 40 Or Convert.ToInt64(b(6)) << 48 Or Convert.ToInt64(b(7)) << 56
            scale = Convert.ToInt64(b(0)) Or Convert.ToInt64(b(1)) << 8L Or
                    Convert.ToInt64(b(2)) << 16L Or Convert.ToInt64(b(3)) << 24L Or
                    Convert.ToInt64(b(4)) << 32L Or Convert.ToInt64(b(5)) << 40L Or
                    Convert.ToInt64(b(6)) << 48L Or Convert.ToInt64(b(7)) << 56L
         End While

         Return Math.Abs(scale)
      End Function

      ''' <summary>
      ''' Random Int64 in Range min - max (without Exceptions)
      ''' </summary>
      ''' <param name="_min">Least legal value for the Random number.</param>
      ''' <param name="_max">The greatest legal return value.</param>
      ''' <returns>Return a Int32 [min - max]</returns>
      Friend Shared Function NextI64(ByVal _min As Int64, ByVal _max As Int64) As Int64
         ' Crypto-random number of Int64 with min - max
         If _min = _max Then Return _min
         SwapCheck(_min, _max)
         Dim scale As Int64 = NextI64()
         Return Math.Abs(CLng(_min + (_max - _min) * (scale / Int64.MaxValue)))
      End Function

      ''' <summary>
      ''' Swaps two IntNum if necessary
      ''' </summary>
      ''' <typeparam name="T">Generic Datatyp</typeparam>
      ''' <param name="_min">Least legal value.</param>
      ''' <param name="_max">Greater legal value.</param>
      Private Shared Sub SwapCheck(Of T As IComparable(Of T))(ByRef _min As T, ByRef _max As T)
         ' Swaps two Variable of T
         If _min.CompareTo(_max) > 0 Then
            Dim tmp As T
            tmp = _min
            _min = _max
            _max = tmp
         End If
      End Sub

      ''' <summary>
      ''' Dient der Warmphase, und kommt
      ''' beim Ersten Nutzen der Klasse
      ''' zum Zug.
      ''' </summary>
      Private Shared Sub WarmUp()
         FirstWarmUp = False
         Dim startcount = CInt(NextI64() Mod 1000)
         For i As Int32 = 0 To startcount - 1
            NextI64()
         Next
      End Sub

      ''' <summary>
      ''' Private CTor
      ''' </summary>
      Private Sub New()
      End Sub
   End Class
End Namespace
