﻿Option Strict On
Option Explicit On
Imports System.IO
Imports System.Drawing
Imports System.Reflection
Imports exc.jdbi.Pixelmanipulation
Imports VBPM = PixelManipulationVb.exc.jdbi.vb.Pixelmanipulation.PixelManipulation

Public Module PixelManipulationTest
   Public Sub Main()
      CopyImagePic1()
      TestParallelCS()
      TestParallelVB()

      Console.WriteLine()
      Console.WriteLine("Finish")
      Console.ReadLine()
   End Sub
   Private Sub TestParallelCS()
      TestGreyScaleParallelCS()
      TestRngBitmapParallelCS()
      TestFindInBitmapParallelCS()
   End Sub
   Private Sub TestParallelVB()
      TestGreyScaleNaiveVB()
      TestGreyScaleParallelVB()
      TestRngBitmapParallelVB()
      TestFindInBitmapParallelVB()
   End Sub
   Private Sub TestGreyScaleNaiveVB()
      Dim exedir = Path.GetDirectoryName(Assembly.GetEntryAssembly.Location)
      Dim src = Path.Combine(exedir, "pic1.png")
      Dim dst = Path.Combine(exedir, "greyvb1.png")
      Dim source = VBPM.ToBitmapFile(src)
      Dim pm = New VBPM


      Dim sw = New Stopwatch
      sw.Restart()

      For y As Int32 = 0 To source.Height - 1
         For x As Int32 = 0 To source.Width - 1
            Dim current = source.GetPixel(x, y)
            Dim avg = CByte((CInt(current.R) + current.B + current.G) / 3)
            Dim output As Color = Color.FromArgb(avg, avg, avg)
            source.SetPixel(x, y, output)
         Next
      Next
      Console.WriteLine($"{NameOf(TestGreyScaleNaiveVB)}: {sw.ElapsedMilliseconds}ms")
      VBPM.SaveBitmap(source, dst, True)
   End Sub
   Private Sub TestGreyScaleParallelVB()
      Dim exedir = Path.GetDirectoryName(Assembly.GetEntryAssembly.Location)
      Dim src = Path.Combine(exedir, "pic1.png")
      Dim dst = Path.Combine(exedir, "greyvb2.png")

      Dim bm = VBPM.ToBitmapFile(src)
      Dim newbm As Bitmap = Nothing
      Dim pm = New VBPM

      Dim sw = New Stopwatch
      sw.Restart()
      Dim t = Task.Factory.StartNew(Sub() newbm = pm.ToGrayScale(bm))
      t.Wait()
      t.Dispose()

      Console.WriteLine($"{NameOf(TestGreyScaleParallelVB)}: {sw.ElapsedMilliseconds}ms")
      VBPM.SaveBitmap(newbm, dst, True)
   End Sub
   Private Sub TestRngBitmapParallelVB()
      Dim width = 1000, height = 750

      Dim exedir = Path.GetDirectoryName(Assembly.GetEntryAssembly.Location)
      Dim dst = Path.Combine(exedir, "rngpicvb.png")

      Dim newbm As Bitmap = Nothing
      Dim pm = New VBPM

      Dim sw = New Stopwatch
      sw.Restart()
      Dim t = Task.Factory.StartNew(Sub() newbm = pm.ToRngBitmap(width, height))
      t.Wait()
      t.Dispose()

      Console.WriteLine($"{NameOf(TestRngBitmapParallelVB)}: {sw.ElapsedMilliseconds}ms")
      PixelManipulation.SaveBitmap(newbm, dst, True)
   End Sub
   Private Sub TestFindInBitmapParallelVB()
      Dim exedir = Path.GetDirectoryName(Assembly.GetEntryAssembly.Location)
      Dim src = Path.Combine(exedir, "rngpicvb.png")

      Dim pm = New VBPM
      Dim cxyval = pm.ToRngPixelColor(src)

      Dim xy() As Point = Nothing
      Dim searchcolor = cxyval.Item1

      Dim sw = New Stopwatch
      sw.Restart()
      Dim t = Task.Factory.StartNew(Sub() xy = pm.FindColor(src, searchcolor, False))
      t.Wait()
      t.Dispose()

      Console.WriteLine($"{NameOf(TestFindInBitmapParallelVB)}: {sw.ElapsedMilliseconds}ms")

      For Each _xy In xy
         Console.WriteLine("{0} at [{1}, {2}]", searchcolor, _xy.X, _xy.Y)
      Next
      Console.WriteLine()
   End Sub

   Private Sub TestGreyScaleParallelCS()
      Dim exedir = Path.GetDirectoryName(Assembly.GetEntryAssembly.Location)
      Dim src = Path.Combine(exedir, "pic1.png")
      Dim dst = Path.Combine(exedir, "greycs.png")

      Dim bm = PixelManipulation.ToBitmapFile(src)

      Dim newbm As Bitmap = Nothing
      Dim pm = New PixelManipulation

      Dim sw = New Stopwatch
      sw.Restart()
      Dim t = Task.Factory.StartNew(Sub() newbm = pm.ToGrayScale(bm))
      t.Wait()
      t.Dispose()

      Console.WriteLine($"{NameOf(TestGreyScaleParallelCS)}: {sw.ElapsedMilliseconds}ms")

      PixelManipulation.SaveBitmap(newbm, dst, True)

   End Sub
   Private Sub TestRngBitmapParallelCS()
      Dim width = 1000, height = 750

      Dim exedir = Path.GetDirectoryName(Assembly.GetEntryAssembly.Location)
      Dim dst = Path.Combine(exedir, "rngpiccs.png")

      Dim newbm As Bitmap = Nothing
      Dim pm = New PixelManipulation

      Dim sw = New Stopwatch
      sw.Restart()
      Dim t = Task.Factory.StartNew(Sub() newbm = pm.ToRngBitmap(width, height))
      t.Wait()
      t.Dispose()

      Console.WriteLine($"{NameOf(TestRngBitmapParallelCS)}: {sw.ElapsedMilliseconds}ms")

      PixelManipulation.SaveBitmap(newbm, dst, True)
   End Sub
   Private Sub TestFindInBitmapParallelCS()
      Dim exedir = Path.GetDirectoryName(Assembly.GetEntryAssembly.Location)
      Dim src = Path.Combine(exedir, "rngpiccs.png")

      Dim pm = New PixelManipulation
      Dim cxyval = pm.ToRngPixelColor(src)

      Dim xy() As Point = Nothing
      Dim searchcolor = cxyval.Item1

      Dim sw = New Stopwatch
      sw.Restart()
      Dim t = Task.Factory.StartNew(Sub() xy = pm.FindColor(src, searchcolor, False))
      t.Wait()
      t.Dispose()

      Console.WriteLine($"{NameOf(TestFindInBitmapParallelCS)}: {sw.ElapsedMilliseconds}ms")

      For Each _xy In xy
         Console.WriteLine("{0} at [{1}, {2}]", searchcolor, _xy.X, _xy.Y)
      Next
      Console.WriteLine()
      Console.WriteLine()
      Console.WriteLine()
   End Sub

   Private Sub CopyImagePic1()
      Dim imgname = "pic1.png"
      Dim exedir = Path.GetDirectoryName(Assembly.GetEntryAssembly.Location)
      Dim di = New DirectoryInfo(exedir)
      Dim sdir = String.Empty
      Dim pdi = di, simg = "Image"
      While True
         pdi = di.Parent
         sdir = Path.Combine(pdi.FullName, simg)
         pdi = New DirectoryInfo(sdir)
         If pdi.Exists Then
            sdir = Path.Combine(pdi.FullName, imgname)
            Dim fi = New FileInfo(sdir)
            If fi.Exists Then
               sdir = Path.Combine(exedir, imgname)
               fi.CopyTo(sdir, True)
            End If
            Return
         End If
         di = pdi.Parent
      End While
   End Sub
End Module
